package com.Smartlook.Smartlook.flutter_smartlook;

import android.os.Handler;
import android.os.Looper;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import io.flutter.Log;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.EventChannel.StreamHandler;
import io.flutter.plugin.common.EventChannel.EventSink;
import io.flutter.plugin.common.MethodChannel.Result;

import com.smartlook.sdk.smartlook.Smartlook;
import com.smartlook.sdk.smartlook.core.bridge.BridgeInterface;
import com.smartlook.sdk.smartlook.core.bridge.WireframeDataCallback;
import com.smartlook.sdk.smartlook.core.bridge.model.WireframeData;
import com.smartlook.sdk.smartlook.integration.IntegrationListener;

import com.google.gson.Gson;
import com.smartlook.sdk.smartlook.util.logging.annotation.LogAspect;

import java.util.ArrayList;

/**
 * SmartlookPlugin
 */
public class SmartlookPlugin implements FlutterPlugin {
    private static final String CHANNEL_NAME = "smartlook";
    private MethodChannel channel;
    private MethodCallHandlerImpl handler;
    private Gson gson = null;

    @SuppressWarnings("deprecation")
    public static void registerWith(io.flutter.plugin.common.PluginRegistry.Registrar registrar) {
        final SmartlookPlugin pluginInstance = new SmartlookPlugin();
        pluginInstance.setupChannel(registrar.messenger());
    }


    private void setupChannel(BinaryMessenger messenger) {
        channel = new MethodChannel(messenger, CHANNEL_NAME);
        handler = new MethodCallHandlerImpl();
        channel.setMethodCallHandler(handler);

        if (gson == null) {
            gson = new Gson();
        }
        ArrayList<LogAspect> logAspects = new ArrayList<LogAspect>();
        logAspects.add(LogAspect.BRIDGE_WIREFRAME);
        //logAspects.add(LogAspect.ALL);

        Smartlook.enableLogging(gson.toJson(logAspects).toString());

        final EventChannel eventChannel = new EventChannel(messenger, "smartlookEvent");
        eventChannel.setStreamHandler(new StreamHandler() {
            @Override
            public void onListen(Object arguments, final EventSink eventSink) {
                Smartlook.registerIntegrationListener(new IntegrationListener() {
                    @Override
                    public void onSessionReady(String dashboardSessionUrl) {
                        eventSink.success(dashboardSessionUrl);
                    }

                    @Override
                    public void onVisitorReady(String dashboardVisitorUrl) {
                        eventSink.success(dashboardVisitorUrl);
                    }
                });

            }

            @Override
            public void onCancel(Object args) {
                Smartlook.unregisterIntegrationListener();
            }
        });

        Smartlook.registerBridgeInterface(new BridgeInterface() {
            @Override
            public void obtainWireframeData(final WireframeDataCallback callback) {
                new Handler(Looper.getMainLooper()).post(new Runnable() {
                    @Override
                    public void run() {
                        channel.invokeMethod("obtainWireframeData", null, new Result() {
                            @Override
                            public void success(@Nullable Object o) {
                                //Log.d("wireframe", o.toString());
                                WireframeData data = gson.fromJson(o.toString(), WireframeData.class);

                                callback.onSuccess(data);
                            }

                            @Override
                            public void error(String s, @Nullable String s1, @Nullable Object o) {
                                Log.d("wireframe", s);
                            }

                            @Override
                            public void notImplemented() {
                                Log.d("wireframe", "not implemented");
                            }
                        });
                    }
                });


            }
        });
    }

    private void teardownChannel() {
        handler.teardown();
        handler = null;
        channel.setMethodCallHandler(null);
        channel = null;
    }


    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding binding) {
        setupChannel(binding.getBinaryMessenger());
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        Smartlook.unregisterIntegrationListener();
    }


}
